import 'package:fis_jsonrpc/services/followUp.m.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:vnoteapp/architecture/defines.dart';
import 'package:vnoteapp/managers/interfaces/follow_up.dart';
import 'package:vnoteapp/pages/check/follow_up/controller.dart';
import 'package:vnoteapp/pages/check/follow_up/widgets/follow_up_from.dart';
import 'package:vnoteapp/pages/check/follow_up_record/state.dart';
import 'package:vnoteapp/pages/check/widgets/configurable_card.dart';

class FollowUpRecordController extends FControllerBase {
  FollowUpRecordController();
  final state = FollowUpRecordState();
  final _followUpManager = Get.find<IFollowUpManager>();
  late String patientCode;
  late String patientName;
  late FollowUpController _followUpController;
  @override
  void onReady() {
    super.onReady();
    _initData();
  }

  _initData() {
    update(["contract_records"]);
    final p = Get.parameters;
    patientCode = p['patientCode'] ?? "";
    patientName = p['patientName'] ?? "";
    _getFollowUpRecordList();
    _followUpController = Get.put(FollowUpController());
  }

  Future<void> _getFollowUpRecordList() async {
    try {
      var result = await _followUpManager.getFollowUpRecordList(
          followUpKeyValue.keys.toList(), patientCode);
      if (result != null) {
        state.followUpDTOList = result;
      }
    } catch (e) {
      return;
    }
  }

  toCheckPage(FollowUpRecordDataDTO dataDTO) async {
    _followUpController.state.followUpTime = dataDTO.followUpTime;
    _followUpController.state.nextFollowUpTime = dataDTO.nextFollowUpTime;
    _followUpController.state.followUpMode = dataDTO.followUpMode;
    Get.to(
      ConfigurableCard(
        cardKey: dataDTO.key!,
        examData: dataDTO.followUpData,
        followUpWidget: const FollowUpFrom(),
        patientCode: patientCode,
        callBack: (key, templateCode, data) async {
          // await controller.createExam(key, templateCode, data);
          // Get.back();
        },
      ),
      transition: Transition.rightToLeft,
    );
  }

  final Map<String, String> followUpKeyValue = {
    'FJHSFFW': "肺结核随访服务",
    'FJHRHSF': "肺结核入户随访",
    'GXB': "冠心病",
    'NCZ': "脑卒中",
    'YZJSZASFFW': "严重精神障碍随访服务",
    'YZJSZAGRXXBC': "严重精神障碍个人信息补充",
    'TNB': "2型糖尿病",
    'GXY': "高血压",
    'LNRZYYJKGLFWJL': "老年人中医药健康管理服务记录表",
    'LNRSHZLNLPGB': "老年人生活自理能力评估表",
    'YCF_CH42TJKJCLB': "产后42天健康检查列表",
    'YCF_CHFSLB': "产后访视列表",
    'YCF_2_5CCQSFLB': "2~5次产前随访列表",
    'YCF_DYCCQJCLB': "第一次产前检查列表",
    'YCF_JBXX': "基本信息",
    'ET_ZYYJKGLLB': "儿童中医药健康管理列表",
    'ET_3_6SETJKJCLB': "3~6岁儿童健康检查列表",
    'ET_1_2SETJKJCLB': "1~2岁儿童健康检查列表",
    'ET_1SNETJKJCLB': "1岁内儿童健康检查列表",
    'ET_XSEFSLB': "新生儿访视列表",
  };

  String getFollowUpValueByKey(String key) {
    if (followUpKeyValue[key] != null) {
      return followUpKeyValue[key]!;
    } else {
      return "";
    }
  }

  String followUpStateTransition(FollowUpStateEnum state) {
    switch (state) {
      case FollowUpStateEnum.NoFollowUp:
        return "未随访";
      case FollowUpStateEnum.FollowUpVisit:
        return "已随访";
      case FollowUpStateEnum.Cancelled:
        return "已作废";
      default:
        return "";
    }
  }

  MaterialColor followUpStateColors(FollowUpStateEnum state) {
    switch (state) {
      case FollowUpStateEnum.NoFollowUp:
        return Colors.grey;
      case FollowUpStateEnum.Cancelled:
        return Colors.red;
      case FollowUpStateEnum.FollowUpVisit:
        return Colors.green;
      default:
        return Colors.blue;
    }
  }

  String getFollowUpMode(FollowUpModeEnum? modeEnum) {
    switch (modeEnum) {
      case FollowUpModeEnum.Outpatient:
        return "门诊";
      case FollowUpModeEnum.Phone:
        return "电话";
      case FollowUpModeEnum.Visit:
        return "家庭";
      default:
        return "";
    }
  }
}
