import 'dart:convert';

import 'package:fis_jsonrpc/rpc.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:super_tooltip/super_tooltip.dart';
import 'package:vnoteapp/architecture/storage/text_storage.dart';
import 'package:vnoteapp/architecture/utils/prompt_box.dart';
import 'package:vnoteapp/components/appbar.dart';
import 'package:vnoteapp/components/cell.dart';
import 'package:vnoteapp/components/checkbox_cell/drawer.dart';
import 'package:vnoteapp/components/checkbox_cell/group.dart';
import 'package:vnoteapp/pages/check/widgets/exam_configurable/dialog_blood_pressure.dart';
import 'package:vnoteapp/components/dialog_input.dart';
import 'package:vnoteapp/components/dialog_number.dart';
import 'package:vnoteapp/components/dialog_select.dart';
import 'package:vnoteapp/components/dynamic_drawer.dart';
import 'package:vnoteapp/managers/interfaces/template.dart';
import 'package:vnoteapp/pages/check/models/data.dart';
import 'package:vnoteapp/pages/check/models/form.dart';

class ConfigurableForm extends StatefulWidget {
  const ConfigurableForm({
    super.key,
    required this.formValueMap,
    required this.menuList,
    required this.callBack,
  });

  final Map<String, dynamic> formValueMap;
  final List<MenuItem> menuList;
  final Function(dynamic, dynamic) callBack;
  @override
  State<ConfigurableForm> createState() => _ConfigurableFormState();
}

class _ConfigurableFormState extends State<ConfigurableForm> {
  String currentSelected = "";
  String currentSelectedLabel = "";
  Map<String, dynamic> formValue = {};
  late List<MenuItem> menuList = widget.menuList;

  /// 当前模板
  List<FormObject> currentTemplate = [];
  var scaffoldKey = GlobalKey<ScaffoldState>();

  Map<String, dynamic> templateRelation = {};

  int _currentIndex = 0;
  final _templateManager = Get.find<ITemplateManager>();

  @override
  void initState() {
    super.initState();
    fetchTemplateIndex();
    currentSelected = menuList[_currentIndex].value;
    currentSelectedLabel = menuList[_currentIndex].label;
    widget.formValueMap.forEach((key, value) {
      formValue[key] = {};
    });
  }

  Future<void> fetchTemplateIndex() async {
    try {
      /// 获取模板的键值对
      String? templates;
      templates = await _templateManager.readTemplate('templateRelation');
      templateRelation = jsonDecode(templates!);
      fetchTemplate(currentSelected);
    } catch (error) {
      print('发生错误: $error');
    }
  }

  Future<void> fetchTemplate(String key) async {
    try {
      if (templateRelation[key] == null) {
        currentTemplate = [];
        setState(() {});
        return;
      }
      var b = await _templateManager.readTemplate(templateRelation[key]!);
      String templateContent =
          TemplateDTO.fromJson(jsonDecode(b!)).templateContent!;
      var list = jsonDecode(templateContent).cast<Map<String, dynamic>>();
      if (list == null) {
        currentTemplate = [];
      } else {
        currentTemplate = updateChildren(list);
      }

      TextStorage t = TextStorage(
        fileName: key,
        directory: "template",
      );
      await t.save(jsonEncode(currentTemplate));
      setState(() {});
    } catch (error) {
      print('发生错误: $error');
    }
  }

  @override
  void dispose() {
    super.dispose();
  }

  /// 下拉选择值改变
  selectChange(value) {
    currentSelected = value;
    currentSelectedLabel =
        menuList.firstWhere((element) => element.value == value).label;
    _currentIndex = menuList.indexWhere((element) => element.value == value);
    _controller.hideTooltip();
    fetchTemplate(currentSelected);
  }

  final _controller = SuperTooltipController();

  @override
  Widget build(BuildContext context) {
    return Scaffold(
        key: scaffoldKey,
        backgroundColor: const Color.fromRGBO(238, 238, 238, 1),
        endDrawer: VDynamicDrawerWrapper(scaffoldKey: scaffoldKey),
        appBar: VAppBar(
          titleWidget: GestureDetector(
            onTap: () async {
              await _controller.showTooltip();
            },
            child: SuperTooltip(
              showBarrier: true,
              controller: _controller,
              hasShadow: false,
              content: SelectWidget(
                items: menuList,
                value: currentSelected,
                valueChanged: selectChange,
              ),
              child: Center(
                child: Row(
                  mainAxisSize: MainAxisSize.max,
                  mainAxisAlignment: MainAxisAlignment.center,
                  crossAxisAlignment: CrossAxisAlignment.center,
                  children: [
                    Text('体检 - $currentSelectedLabel'),
                    const Icon(
                      Icons.keyboard_arrow_down_rounded,
                      color: Colors.white,
                    ),
                  ],
                ),
              ),
            ),
          ),
          actions: [
            if (_currentIndex != 0)
              TextButton(
                onPressed: () {
                  _currentIndex--;
                  selectChange(menuList[_currentIndex].value);
                },
                style: ButtonStyle(
                  padding: MaterialStateProperty.all<EdgeInsets>(
                    const EdgeInsets.symmetric(
                      vertical: 25,
                      horizontal: 15,
                    ), // 设置填充的大小
                  ),
                ),
                child: const Text(
                  '上一步',
                  style: TextStyle(color: Colors.white, fontSize: 20),
                ),
              ),
            if (_currentIndex != menuList.length - 1)
              TextButton(
                onPressed: () {
                  _currentIndex++;
                  selectChange(menuList[_currentIndex].value);
                },
                style: ButtonStyle(
                  padding: MaterialStateProperty.all<EdgeInsets>(
                    const EdgeInsets.symmetric(
                      vertical: 25,
                      horizontal: 15,
                    ), // 设置填充的大小
                  ),
                ),
                child: const Text(
                  '下一步',
                  style: TextStyle(color: Colors.white, fontSize: 20),
                ),
              ),
            if (_currentIndex == menuList.length - 1)
              TextButton(
                onPressed: () {
                  widget.callBack.call(formValue, templateRelation);
                  Get.back();
                  PromptBox.toast('提交成功');
                },
                style: ButtonStyle(
                  padding: MaterialStateProperty.all<EdgeInsets>(
                    const EdgeInsets.symmetric(
                      vertical: 25,
                      horizontal: 15,
                    ), // 设置填充的大小
                  ),
                ),
                child: const Text(
                  '提交',
                  style: TextStyle(color: Colors.white, fontSize: 20),
                ),
              ),
            const SizedBox(
              width: 15,
            ),
          ],
        ),
        body: Stack(
          children: [
            Scrollbar(
              thumbVisibility: true,
              child: SingleChildScrollView(
                child: Container(
                  alignment: Alignment.topCenter,
                  margin: const EdgeInsets.only(bottom: 64),
                  child: SizedBox(
                    width: 800,
                    child: Column(
                      children: currentTemplate.map((e) {
                        if (e.type == 'checkbox') {
                          return _buildCheckbox(e);
                        } else if (e.type == 'title' ||
                            e.type == 'virtualTitle') {
                          return _buildTitle(e);
                        } else {
                          return _buildInput(e);
                        }
                      }).toList(),
                    ),
                  ),
                ),
              ),
            ),
            if (_currentIndex != menuList.length - 1)
              Positioned.fill(
                right: 50,
                child: Align(
                  alignment: Alignment.centerRight,
                  child: SizedBox(
                    width: 90,
                    height: 90,
                    child: ElevatedButton(
                      style: ButtonStyle(
                        shape: MaterialStatePropertyAll(
                          RoundedRectangleBorder(
                            borderRadius: BorderRadius.circular(45),
                            side: BorderSide(
                              color: Theme.of(context).primaryColor,
                            ),
                          ),
                        ),
                      ),
                      onPressed: () async {
                        widget.callBack.call(formValue, templateRelation);
                        Get.back();
                        PromptBox.toast('提交成功');
                      },
                      child: const Text(
                        "提交",
                        style: TextStyle(fontWeight: FontWeight.bold),
                      ),
                    ),
                  ),
                ),
              ),
          ],
        ));
  }

  /// 多选框组件
  Widget _buildCheckbox(FormObject checkbox) {
    List<CheckBoxCellGroupItem<Map>> options = [];
    List<String> labelList = [];
    var templateContent = formValue[currentSelected];
    for (Option i in checkbox.options!) {
      options.add(
        CheckBoxCellGroupItem(
          i.toJson(),
          isChecked: templateContent[checkbox.key]?.contains(i.value) ?? false,
        ),
      );
    }
    for (var element in options) {
      if (templateContent[checkbox.key] != null &&
          templateContent[checkbox.key]?.contains(element.data['value'])) {
        labelList.add(element.data['label']);
      }
    }
    return Column(
      children: [
        const SizedBox(
          height: 20,
        ),
        VListFormCellGroup(
          children: [
            VListFormCell(
              label: checkbox.label,
              content: labelList.join('、'),
              onTap: () {
                VDynamicDrawerWrapper.show(
                  scaffoldKey: scaffoldKey,
                  builder: (_) {
                    return VDrawerCheckBoxCellGroup(
                      title: checkbox.label,
                      source: options,
                      labelGetter: (data) => data['label'],
                      valueGetter: (data) => data['value'],
                      onChanged: (value, isChecked, data, checkedValues) {},
                      onConfirm: (value) {
                        templateContent[checkbox.key] = value;
                        setState(() {});
                      },
                      scaffoldKey: scaffoldKey,
                    );
                  },
                );
              },
            ),
          ],
        ),
      ],
    );
  }

  /// 单选框组件
  Widget _buildRadio(FormObject radio) {
    List<VSelectModel> options = [];
    String labelList = '';
    var templateContent = formValue[currentSelected];
    for (Option i in radio.options ?? []) {
      options.add(
        VSelectModel(code: i.value!, name: i.label!),
      );
    }
    for (VSelectModel element in options) {
      if (templateContent[radio.key] != null &&
          templateContent[radio.key]?.contains(element.code)) {
        labelList = element.name;
      }
    }
    return VListFormCell(
      label: radio.label,
      content: labelList,
      onTap: () async {
        String? result = await VDialogSelect<VSelectModel, String>(
          title: radio.label,
          source: options,
          labelGetter: (data) => data.name,
          valueGetter: (data) => data.code,
          initialValue: templateContent[radio.key],
        ).show();
        templateContent[radio.key] = result ?? templateContent[radio.key] ?? '';
        setState(() {});
      },
    );
  }

  /// 单选框分数组件
  Widget _buildRadioScore(FormObject radio) {
    List<VSelectModel> options = [];
    String labelList = '';
    var templateContent = formValue[currentSelected];
    for (Option i in radio.options ?? []) {
      options.add(
        VSelectModel(code: i.value!, name: i.label!),
      );
    }
    for (VSelectModel element in options) {
      if (templateContent[radio.childrenKey?.first] != null &&
          templateContent[radio.childrenKey?.first]?.contains(element.code)) {
        labelList = element.name;
      }
    }
    return VListFormCell(
      label: radio.label,
      content: labelList,
      onTap: () async {
        String? result = await VDialogSelect<VSelectModel, String>(
          title: radio.label,
          source: options,
          labelGetter: (data) => data.name,
          valueGetter: (data) => data.code,
          initialValue: templateContent[radio.childrenKey?.first],
        ).show();
        templateContent[radio.childrenKey?.first] =
            result ?? templateContent[radio.childrenKey?.first] ?? '';
        setState(() {});
      },
    );
  }

  /// 标题块级组件
  Widget _buildTitle(FormObject title) {
    return Column(
      children: [
        const SizedBox(
          height: 20,
        ),
        if (title.children?.isNotEmpty ?? false)
          VListFormCellGroup(
            children: (title.children as List<FormObject>).map((item) {
              if (item.type == 'bloodPressure') {
                return _buildBloodPressure(item);
              } else if (item.type == 'input') {
                return _buildInput(item);
              } else if (item.type == 'numberInput') {
                return _buildNumberInput(item);
              } else if (item.type == "radio") {
                return _buildRadio(item);
              } else if (item.type == "radioScore") {
                return _buildRadioScore(item);
              } else {
                return const SizedBox();
              }
            }).toList(),
          ),
      ],
    );
  }

  /// 血压输入框
  Widget _buildBloodPressure(FormObject bloodPressure) {
    var templateContent = formValue[currentSelected];
    String? value1 = templateContent[bloodPressure.childrenKey?.first];
    String? value2 = templateContent[bloodPressure.childrenKey?.last];
    bool isShowDividing = false;
    if (value1 != null && value1 != '' && value2 != null && value2 != '') {
      isShowDividing = true;
    }
    return VListFormCell(
      label: bloodPressure.label,
      contentWidget: Expanded(
        child: Row(
          children: [
            const SizedBox(
              width: 150,
            ),
            Expanded(
              child: Text(
                value1 ?? "",
                textAlign: TextAlign.end,
                style: const TextStyle(
                  fontSize: 20,
                ),
              ),
            ),
            const SizedBox(
              width: 10,
            ),
            if (isShowDividing)
              const Text(
                '/',
                style: TextStyle(fontSize: 20),
              ),
            const SizedBox(
              width: 10,
            ),
            Expanded(
              child: Text(
                value2 ?? "",
                style: const TextStyle(
                  fontSize: 20,
                ),
              ),
            ),
            if (bloodPressure.buttonName?.isNotEmpty ?? false)
              OutlinedButton(
                onPressed: () {},
                child: Text(bloodPressure.buttonName!),
              )
          ],
        ),
      ),
      onTap: () async {
        String? result = await VDialogBloodPressure(
          title: bloodPressure.label,
          placeholder: '请输入',
          initialValue: [value1 ?? "", value2 ?? ""],
        ).show();
        if (result?.isNotEmpty ?? false) {
          List resultList = jsonDecode(result!);
          setState(() {
            templateContent[bloodPressure.childrenKey?.first] =
                resultList.first;
            templateContent[bloodPressure.childrenKey?.last] = resultList.last;
          });
        }
      },
    );
  }

  /// input输入组件
  Widget _buildInput(FormObject input) {
    var templateContent = formValue[currentSelected];

    var value = templateContent[input.key];
    if (value != null) {
      value = templateContent[input.key] + ' ' + input.append ?? "";
    } else {
      value = input.append ?? "";
    }
    return VListFormCell(
      label: input.label,
      content: value,
      onTap: () async {
        String? result = await VDialogInput(
          title: input.label,
          initialValue: templateContent[input.key],
        ).show();
        if (result?.isNotEmpty ?? false) {
          templateContent[input.key] = result;
          setState(() {});
        }
      },
    );
  }

  /// numberInput输入组件
  Widget _buildNumberInput(FormObject numberInput) {
    var templateContent = formValue[currentSelected];
    var value = templateContent[numberInput.key];
    if (value != null) {
      value = templateContent[numberInput.key] + ' ' + numberInput.append ?? "";
    } else {
      value = numberInput.append ?? "";
    }
    return VListFormCell(
      label: numberInput.label,
      // content: value,
      contentWidget: Expanded(
        child: Row(
          mainAxisAlignment: MainAxisAlignment.end,
          children: [
            Text(
              value,
              style: const TextStyle(fontSize: 20),
            ),
            const SizedBox(
              width: 20,
            ),
            if (numberInput.buttonName?.isNotEmpty ?? false)
              OutlinedButton(
                onPressed: () {},
                child: Text(numberInput.buttonName!),
              )
          ],
        ),
      ),
      onTap: () async {
        String? result = await VDialogNumber(
          title: numberInput.label,
          initialValue: formValue[numberInput.key],
        ).show();
        if (result?.isNotEmpty ?? false) {
          templateContent[numberInput.key] = result;
          setState(() {});
        }
      },
    );
  }
}

class MenuItem {
  // 显示的文本
  String label;
  // 选中的值
  dynamic value;
  // 是否选中
  bool checked;

  MenuItem({this.label = '', this.value, this.checked = false});
}

class SelectWidget extends StatefulWidget {
  // 显示的菜单项
  final List<MenuItem> items;
  // 当前选中的值
  final dynamic value;
  // 选择框前的标题
  final String? title;
  // 提示语
  final String tooltip;
  // 选中数据的回调事件
  final ValueChanged<dynamic>? valueChanged;
  const SelectWidget({
    Key? key,
    this.items = const [],
    this.value,
    this.valueChanged,
    this.title,
    this.tooltip = "点击选择",
  }) : super(key: key);

  @override
  State<SelectWidget> createState() => _SelectWidgetState();
}

class _SelectWidgetState extends State<SelectWidget> {
  String label = '请选择';
  // 是否展开下拉按钮
  bool isExpand = false;
  // 当前的值
  dynamic formValue;

  @override
  void initState() {
    formValue = widget.value;
    super.initState();
  }

  /// 根据当前的value处理当前文本显示
  void initTitle() {
    if (formValue != null) {
      // 有值查值
      for (MenuItem item in widget.items) {
        if (item.value == formValue) {
          label = item.label;
          return;
        }
      }
    }
    // 没值默认取第一个
    if (widget.items.isNotEmpty) {
      label = widget.items[0].label;
    }
  }

  @override
  Widget build(BuildContext context) {
    initTitle();
    return Wrap(
      children: [
        if (widget.title != null)
          Text(
            widget.title!,
            style: const TextStyle(
              fontSize: 20,
            ),
          ),
        Material(
          child: Column(
            children: widget.items
                .map(
                  (item) => item.value == formValue
                      ? InkWell(
                          child: Container(
                            padding: const EdgeInsets.symmetric(
                              horizontal: 20,
                              vertical: 10,
                            ),
                            child: Text(
                              item.label,
                              style: TextStyle(
                                color: Theme.of(context).primaryColor,
                                fontSize: 18,
                              ),
                            ),
                          ),
                        )
                      : InkWell(
                          child: Container(
                            padding: const EdgeInsets.symmetric(
                              horizontal: 20,
                              vertical: 10,
                            ),
                            child: Text(
                              item.label,
                              style: const TextStyle(
                                fontSize: 18,
                              ),
                            ),
                          ),
                          onTap: () {
                            widget.valueChanged!.call(item.value);
                          },
                        ),
                )
                .toList(),
          ),
        )
      ],
    );
  }
}
